/*
 * pathnames.h
 *
 * dsniff pathnames.
 *
 * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
 *
 * $Id: pathnames.h,v 1.2 2001/03/15 08:33:06 dugsong Exp $
 */

#ifndef PATHNAMES_H
#define PATHNAMES_H

#ifndef DSNIFF_LIBDIR
#define DSNIFF_LIBDIR		"/usr/share/dsniff/"
#endif

#define DSNIFF_SERVICES		"dsniff.services"
#define DSNIFF_MAGIC		"dsniff.magic"

#endif /* PATHNAMES_H */
