/*
 * options.h
 *
 * Global options.
 *
 * Copyright (c) 2000 Dug Song <dugsong@monkey.org>
 *
 * $Id: options.h,v 1.4 2001/03/15 08:33:06 dugsong Exp $
 */

#ifndef OPTIONS_H
#define OPTIONS_H

extern int	Opt_client;
extern u_short	Opt_dns;
extern int	Opt_debug;
extern int	Opt_magic;
extern int	Opt_read;
extern int	Opt_write;
extern int	Opt_snaplen;
extern int	Opt_lines;
extern int	Opt_verbose;
extern int	Opt_show_dups;
extern int	Opt_color;

#endif /* OPTIONS_H */

